package com.javacodegeeks.quarkus.reservation;

import java.util.Collection;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.enterprise.context.ApplicationScoped;
import javax.transaction.Transactional;
import javax.validation.Valid;

@ApplicationScoped
public class ReservationService {
    @Transactional
    public Collection<Reservation> reservations() {
        List<ReservationEntity> list = ReservationEntity.listAll();
        return list
            .stream()
            .map(this::convert)
            .collect(Collectors.toList());
    }

    @Transactional
    public Reservation reserve(@Valid CreateReservation payload) {
        final ReservationEntity entity = convert(payload);
        ReservationEntity.persist(entity);
        return convert(entity);
    }
    
    private ReservationEntity convert(CreateReservation source) {
        final ReservationEntity entity = new ReservationEntity(UUID.randomUUID(), source.getVehicleId());
        entity.setFrom(source.getFrom());
        entity.setTo(source.getTo());
        entity.setStatus(Status.CREATED);
        return entity;
    }
    
    private Reservation convert(ReservationEntity source) {
        final Reservation reservation = new Reservation(source.getId());
        reservation.setVehicleId(source.getVehicleId());
        reservation.setFrom(source.getFrom());
        reservation.setTo(source.getTo());
        reservation.setStatus(source.getStatus().name());
        return reservation;
    }
}
