package com.javacodegeeks.spring.reservation;

import java.util.Collection;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ReservationService {
    private final ReservationRepository reservationRepository;
    
    @Autowired
    public ReservationService(ReservationRepository reservationRepository) {
        this.reservationRepository = reservationRepository;
    }
    
    public Collection<Reservation> reservations() {
        return StreamSupport
            .stream(reservationRepository.findAll().spliterator(), false)
            .map(this::convert)
            .collect(Collectors.toList());
    }

    public Reservation reserve(@Valid CreateReservation payload) {
        return convert(reservationRepository.save(convert(payload)));
    }
    
    private ReservationEntity convert(CreateReservation source) {
        final ReservationEntity entity = new ReservationEntity(UUID.randomUUID(), source.getVehicleId());
        entity.setFrom(source.getFrom());
        entity.setTo(source.getTo());
        entity.setStatus(Status.CREATED);
        return entity;
    }
    
    private Reservation convert(ReservationEntity source) {
        final Reservation reservation = new Reservation(source.getId());
        reservation.setVehicleId(source.getVehicleId());
        reservation.setFrom(source.getFrom());
        reservation.setTo(source.getTo());
        reservation.setStatus(source.getStatus().name());
        return reservation;
    }
}
